/*
 * Copyright (c) 2018, salesforce.com, inc.
 * All rights reserved.
 * SPDX-License-Identifier: MIT
 * For full license text, see the LICENSE file in the repo root or https://opensource.org/licenses/MIT
 */
import * as http from 'http';
import * as https from 'https';
import { URL } from 'node:url';
import { WebSocketServer } from 'ws';
import { LogLevel } from '../../types/index.js';
export class Connection {
    constructor(protocol, host, port, logger, httpsConfig) {
        this.protocol = protocol;
        this.host = host;
        this.port = port;
        this.logger = logger;
        this.httpsConfig = httpsConfig;
    }
    sendModule(modulePath, src) {
        this.send({
            type: 'module-update',
            data: [{ modulePath, src }],
        });
    }
    initializeConnection(initCallback, messageCallback) {
        const httpServer = this.httpsConfig
            ? https.createServer(this.httpsConfig)
            : http.createServer();
        httpServer.on('request', requestHandler);
        try {
            const wss = new WebSocketServer({ server: httpServer });
            wss.on('connection', (ws) => {
                initCallback();
                this.socket = ws;
                this.socket.addEventListener('message', ({ data }) => {
                    if (data && typeof data === 'string') {
                        // When there is an update, handle the incoming message and request an updated module
                        messageCallback(JSON.parse(data));
                    }
                });
            });
            httpServer.listen(this.port);
            this.logger.log(LogLevel.info, `LWC Dev Server started at port:${this.port}`);
        }
        catch (e) {
            this.logger.log(LogLevel.error, `Failed to start a WebSocket server with the following error: ${e}`);
            process.exit(1);
        }
    }
    send(data) {
        if (this.socket && this.socket.readyState === this.socket.OPEN) {
            this.socket.send(JSON.stringify(data));
        }
    }
    close() {
        this.socket?.close();
    }
}
function requestHandler(req, res) {
    // the second arg could really be anything, we're just interested in searchParams
    // also req.url can never be undefined, it's just bad typing
    const url = new URL(req.url, 'http://localhost');
    const redirectUrl = url.searchParams.get('redirectUrl');
    if (redirectUrl) {
        res.statusCode = 302;
        res.setHeader('Location', redirectUrl);
        return res.end();
    }
    res.writeHead(200, { 'Content-Type': 'text/html' });
    res.end(`Visit https://&lt;salesforce-url&gt;?aura.mode=devpreview`);
}
//# sourceMappingURL=index.js.map