import fs from 'fs';
import path from 'path';
import { LogLevel } from '../types/index.js';
import { startWatch } from './watcher/index.js';
import { Connection } from './connection/index.js';
import { compile } from './bundler/lexBundler.js';
import { ModuleGraph } from './moduleGraph/index.js';
import ConsoleLogger from './consoleLogger.js';
import ClientConnection, { clientLookup } from './connection/clientConnection.js';
export class LWCServer {
    constructor(config) {
        this.clients = [];
        this.logger = new ConsoleLogger(config.logLevel);
        this.config = config;
        this.watcher = startWatch(this.config.paths);
        this.connection = new Connection(this.config.protocol, this.config.host, this.config.port, this.logger, this.config.https);
        this.connection.initializeConnection(() => { }, this.messageCallback.bind(this));
        this.watcher.on('change', this.changeHandler.bind(this));
        this.moduleGraph = new ModuleGraph(config.workspace, config.paths, this.logger);
    }
    stopServer() {
        this.watcher.close();
        this.connection.close;
    }
    changeHandler(filePath) {
        const changedFile = path.resolve(filePath);
        if (fs.lstatSync(changedFile).isFile()) {
            const modulePath = this.moduleGraph.getModuleByFile(changedFile) ?? changedFile;
            if (modulePath) {
                const hotModule = { type: 'update', data: [{ modulePath }] };
                this.connection.send(hotModule);
            }
        }
    }
    messageCallback(data) {
        switch (data.type) {
            case 'init': {
                const { data: { activePaths, url, target }, } = data;
                // WebSocket will broadcast to all client connections, so hot module updates will be
                // sent to all active connections from a client. This check avoids double registration
                // for a given client url:target combination
                if (clientLookup.has(ClientConnection.getKey(url, target))) {
                    this.logger.log(LogLevel.debug, `Existing connection for url: ${url} and target: ${target}, avoiding duplicate registration.`);
                }
                else {
                    const connection = new ClientConnection(url, target);
                    connection.addActivePaths(activePaths.map(({ data: { modulePath, compileOptions } }) => {
                        return { modulePath, compileOptions };
                    }));
                    this.clients.push(connection);
                    this.moduleGraph.registerActivePaths(activePaths);
                    this.logger.log(LogLevel.info, `New connection for url: ${url} and target: ${target} registered.`);
                }
                break;
            }
            // In coming hot module
            case 'fetch': {
                const { data: { modulePath }, } = data;
                const moduleFilePath = this.moduleGraph.getFileByModulePath(modulePath);
                if (!moduleFilePath) {
                    const error = {
                        type: 'error',
                        data: {
                            message: `Fetch of ${modulePath} failed as it does not exist in dev server workspace`,
                        },
                    };
                    this.connection.send(error);
                }
                this.clients.forEach(async (client) => {
                    switch (client.target) {
                        case 'LEX': {
                            const compileOptions = client.getCompileOptions(modulePath);
                            if (!compileOptions) {
                                this.logger.log(LogLevel.debug, `Found no compile option for module path: ${modulePath}, check the __lwc_hmr_hot.register() call.`);
                                break;
                            }
                            const result = await compile(this, modulePath, compileOptions);
                            if (!result.code) {
                                this.logger.log(LogLevel.error, `Compiling ${moduleFilePath} failed and there are compiler warnings: ${JSON.stringify(result.warnings)}`);
                                break;
                            }
                            this.connection.sendModule(modulePath, result.code);
                            break;
                        }
                        default:
                            this.logger.log(LogLevel.verbose, `Client target ${client.target} not supported currently.`);
                            break;
                    }
                });
                break;
            }
        }
    }
}
//# sourceMappingURL=index.js.map